#!/usr/bin/env python3
# Проверка на наличие установленных модулей
try:
    import rbtconfig
except ImportError:
    import sys
    print('Отсутствует необходимый модуль rbtconfig.\n'
          'Для его установки выполните команду: apt-get install python3-rbtconfig.', file=sys.stderr)
    exit(1)
else:
    from rbtconfig import *


class Apache(BaseScript):
    MODE_PAM_REQUIRED_PACKAGE = 'libapache2-mod-auth-pam'
    MODE_ALD_REQUIRED_PACKAGE = 'libapache2-mod-auth-kerb'

    def __init__(self, params):
        service_name = 'apache'
        service_config_dir = '/etc/apache2'
        service_owner = 'www-data'
        available_modes = [MODE_PAM, MODE_ALD]
        params.update({
            EXTENDED_DESCRIPTIONS: {
                MODE_PAM: '%s для сайта \'default\'' % MODE_PAM_EXTENDED_DESCRIPTION,
                MODE_ALD: '%s для сайта \'default\' с параметром safe_credential = on' % MODE_ALD_EXTENDED_DESCRIPTION
            },
            REQUIRED_PACKAGES: {
                MODE_PAM: [self.MODE_PAM_REQUIRED_PACKAGE],
                MODE_ALD: [self.MODE_ALD_REQUIRED_PACKAGE]
            },
            INCOMPATIBLE_MODES: [(MODE_ALD, MODE_PAM)],
            INIT_D_SERVICE_NAME: 'apache2',
            KEYTAB_FILE_NAME: 'keytab',
            KRB_PRINCIPAL_NAME: 'HTTP'
        })
        self.__default_host_file = '/etc/apache2/sites-available/default'
        self.__pam_script = '/etc/pam.d/apache2'
        self.__faillog_path = '/var/log/faillog'
        self.__pam_script_data = ['account required pam_tally.so']
        self.__virtual_host_data = {
            MODE_PAM: [
                'AuthPAM_Enabled on',
                'AuthType Basic',
                'AuthName "PAM authentication"',
                'require valid-user'
            ],
            MODE_ALD: [
                'AuthType Kerberos',
                'KrbAuthRealms %s' % AldUtils.realm(),
                'KrbServiceName HTTP/%s' % BaseUtils.hostname(),
                'Krb5Keytab %s/keytab' % service_config_dir,
                'KrbMethodNegotiate on',
                'KrbMethodK5Passwd off',
                'require valid-user',
                'KrbSaveCredentials on'
            ]
        }
        super().__init__(service_name,
                         service_owner,
                         service_config_dir=service_config_dir,
                         available_modes=available_modes,
                         params=params
                         )

    def a2enmod(self, module):
        return self.local_utils.exec_command('a2enmod %s' % module, enable_stderr=True)

    def a2dismod(self, module):
        return self.local_utils.exec_command('a2dismod %s' % module, enable_stderr=True)

    def check_is_module_loaded(self, module):
        """
        Проверяет, запущен ли модуль :param module
        :param module: str
        :return: bool
        """
        try:
            self.local_utils.exec_command('apache2ctl -t -D DUMP_MODULES | grep %s' % module)
        except RbtConfigError:
            return False
        return True

    def check_pam(self):
        checks = list()
        checks.append(self.local_utils.check_facl_on_parsec_db())
        checks.append(self.local_utils.check_is_user_in_shadow_group())
        config_check = self.config_parser.config_file_contains_rows(self.__default_host_file,
                                                                    ['ServerName %s' % self.local_utils.hostname()])
        if not config_check:
            checks.append(config_check)
        checks.append(self.config_parser.config_file_contains_rows(self.__default_host_file,
                                                                   self.__virtual_host_data[MODE_PAM]))
        auth_module_load = self.check_is_module_loaded('auth_pam')
        if not auth_module_load:
            checks.append(auth_module_load)
        checks.append(self.config_parser.config_file_contains_rows(self.__pam_script, self.__pam_script_data))
        checks.append(self.local_utils.check_facl(self.__faillog_path, 'rw'))
        return analyze_check(checks)

    def configure_pam(self):
        self.exec_function_with_inverse_call(direct=self.local_utils.set_facl_on_parsec_db_to_user,
                                             inverse=self.local_utils.del_facl_on_parsec_db_from_user)
        if self.check_is_module_loaded('auth_kerb'):
            self.exec_function_with_inverse_call(direct=self.a2dismod,
                                                 inverse=self.a2enmod,
                                                 direct_args=['auth_kerb'])
        self.exec_function_with_inverse_call(direct=self.a2enmod,
                                             inverse=self.a2dismod,
                                             direct_args=['auth_pam'])
        self.exec_function_with_inverse_call(direct=self.local_utils.add_user_to_shadow_group,
                                             inverse=self.local_utils.del_user_from_shadow_group)
        self.config_parser.add_after(self.__default_host_file,
                                     ['ServerName %s' % self.local_utils.hostname()],
                                     '<VirtualHost *:80>')
        self.config_parser.add_after(self.__default_host_file,
                                     self.__virtual_host_data[MODE_PAM],
                                     '<Directory /var/www/>')
        self.config_parser.add_after(self.__pam_script, self.__pam_script_data)
        self.exec_function_with_inverse_call(direct=self.local_utils.set_facl,
                                             inverse=self.local_utils.del_facl,
                                             direct_args=['rw', self.__faillog_path],
                                             inverse_args=[self.__faillog_path])

    def reset_pam(self):
        self.exec_function_with_inverse_call(direct=self.local_utils.del_facl_on_parsec_db_from_user,
                                             inverse=self.local_utils.set_facl_on_parsec_db_to_user)
        self.config_parser.remove_rows_from_config(self.__default_host_file,
                                                   ['ServerName %s' % self.local_utils.hostname()])
        self.config_parser.remove_rows_from_config(self.__default_host_file,
                                                   self.__virtual_host_data[MODE_PAM])
        self.exec_function_with_inverse_call(direct=self.local_utils.del_user_from_shadow_group,
                                             inverse=self.local_utils.add_user_to_shadow_group)
        self.exec_function_with_inverse_call(direct=self.a2enmod,
                                             inverse=self.a2dismod,
                                             direct_args=['auth_pam'])
        self.config_parser.remove_rows_from_config(self.__pam_script, self.__pam_script_data)
        self.exec_function_with_inverse_call(direct=self.local_utils.del_facl,
                                             inverse=self.local_utils.set_facl,
                                             direct_args=[self.__faillog_path],
                                             inverse_args=['rw', self.__faillog_path])
        if self.local_utils.check_if_package_exists(self.MODE_ALD_REQUIRED_PACKAGE):
            self.exec_function_with_inverse_call(direct=self.a2enmod,
                                                 inverse=self.a2dismod,
                                                 direct_args=['auth_kerb'])

    def check_ald(self):
        checks = list()
        config_check = self.config_parser.config_file_contains_rows(self.__default_host_file,
                                                                    ['ServerName %s' % self.local_utils.hostname()])
        if not config_check:
            checks.append(config_check)
        checks.append(self.config_parser.config_file_contains_rows(self.__default_host_file,
                                                                   self.__virtual_host_data[MODE_ALD]))
        auth_module_load = self.check_is_module_loaded('auth_kerb')
        if not auth_module_load:
            checks.append(auth_module_load)
        checks.append(self.ald_utils.check_is_exists_keytab())
        if self.ald_utils.check_if_ald_pass_file_exits():
            checks.append(self.ald_utils.check_is_service_in_domain())
            checks.append(self.ald_utils.check_is_service_in_group('mac'))
        return analyze_check(checks)

    def configure_ald(self):
        if self.check_is_module_loaded('auth_pam'):
            self.exec_function_with_inverse_call(direct=self.a2dismod,
                                                 inverse=self.a2enmod,
                                                 direct_args=['auth_pam'])
        self.exec_function_with_inverse_call(direct=self.a2enmod,
                                             inverse=self.a2dismod,
                                             direct_args=['auth_kerb'])
        self.exec_function_with_inverse_call(direct=self.ald_utils.add_service_to_domain,
                                             inverse=self.ald_utils.del_service_from_domain)
        self.exec_function_with_inverse_call(direct=self.ald_utils.add_service_to_group,
                                             inverse=self.ald_utils.del_service_from_group,
                                             direct_args=['mac'])
        self.exec_function_with_inverse_call(direct=self.ald_utils.create_keytab_for_service,
                                             inverse=self.ald_utils.remove_keytab)
        self.config_parser.add_after(self.__default_host_file,
                                     ['ServerName %s' % self.local_utils.hostname()],
                                     '<VirtualHost *:80>')
        self.config_parser.add_after(self.__default_host_file,
                                     self.__virtual_host_data[MODE_ALD],
                                     '<Directory /var/www/>')

    def reset_ald(self):
        self.exec_function_with_inverse_call(direct=self.ald_utils.remove_keytab,
                                             inverse=self.ald_utils.create_keytab_for_service)
        self.exec_function_with_inverse_call(direct=self.ald_utils.del_service_from_group,
                                             inverse=self.ald_utils.add_service_to_group,
                                             direct_args=['mac'])
        self.exec_function_with_inverse_call(direct=self.ald_utils.del_service_from_domain,
                                             inverse=self.ald_utils.add_service_to_domain)
        self.config_parser.remove_rows_from_config(self.__default_host_file,
                                                   ['ServerName %s' % self.local_utils.hostname()])
        self.config_parser.remove_rows_from_config(self.__default_host_file,
                                                   self.__virtual_host_data[MODE_ALD])
        self.exec_function_with_inverse_call(direct=self.a2dismod,
                                             inverse=self.a2enmod,
                                             direct_args=['auth_kerb'])
        if self.local_utils.check_if_package_exists(self.MODE_PAM_REQUIRED_PACKAGE):
            self.exec_function_with_inverse_call(direct=self.a2enmod,
                                                 inverse=self.a2dismod,
                                                 direct_args=['auth_pam'])

if __name__ == '__main__':
    run(Apache, param_required=False)
